/* **********************************************************************************
 * Project Name: Verification Environment for Robot Controller.
 * File Name:    golden_model.c
 * Description:  Golden model - maze solver for Robot Controller implementing left hand algorithm.
 * Author:       Stanislav Krajcir <xkrajc06@stud.fit.vutbr.cz>
 * Supervisor:	 Marcela Simkova <isimkova@fit.vutbr.cz>   
 * Date:         25.5.2015
 * **********************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "golden_model.h"



int main()
{


	/* This section of main only demonstrates functionality of maze solver

	set_goal(12, 3);

	//48 times
	const int data[48][8] = {

			{ 0, 149, 125, 38, 4, 4, 12, 4 },
			{ 0, 145, 127, 44, 12, 4, 4, 12 },
			{ 0, 138, 119, 47, 4, 12, 20, 4 },
			{ 0, 134, 123, 54, 4, 4, 12, 12 },
			{ 0, 130, 126, 61, 4, 12, 4, 20 },
			{ 0, 137, 133, 59, 12, 4, 52, 4 },
			{ 0, 134, 136, 66, 4, 4, 44, 12 },
			{ 0, 131, 140, 73, 28, 4, 36, 20 },
			{ 0, 122, 134, 76, 20, 12, 4, 4 },
			{ 0, 115, 127, 79, 12, 20, 4, 4 },
			{ 0, 108, 120, 82, 4, 28, 4, 36 },
			{ 0, 111, 115, 76, 20, 4, 12, 28 },
			{ 0, 115, 111, 69, 4, 4, 20, 20 },
			{ 0, 120, 107, 62, 4, 4, 28, 12 },
			{ 0, 124, 104, 57, 20, 4, 36, 4 },
			{ 0, 118, 97, 62, 12, 12, 4, 4 },
			{ 0, 113, 90, 69, 4, 20, 4, 12 },
			{ 0, 118, 86, 64, 20, 4, 12, 4 },
			{ 0, 113, 79, 71, 12, 12, 4, 4 },
			{ 0, 109, 72, 79, 4, 20, 36, 4 },
			{ 0, 103, 76, 82, 4, 4, 28, 12 },
			{ 0, 96, 80, 86, 4, 4, 20, 20 },
			{ 0, 90, 85, 91, 4, 4, 12, 28 },
			{ 0, 84, 90, 96, 12, 4, 4, 36 },
			{ 0, 79, 85, 103, 4, 12, 20, 4 },
			{ 0, 73, 91, 108, 4, 4, 12, 12 },
			{ 0, 68, 97, 113, 20, 28, 4, 20 },
			{ 0, 62, 92, 120, 12, 36, 4, 4 },
			{ 0, 55, 89, 125, 4, 44, 28, 76 },
			{ 0, 62, 82, 120, 4, 4, 36, 68 },
			{ 0, 69, 75, 116, 4, 4, 44, 60 },
			{ 0, 75, 68, 112, 4, 4, 52, 52 },
			{ 0, 82, 62, 108, 4, 4, 60, 44 },
			{ 0, 89, 56, 105, 4, 4, 68, 36 },
			{ 0, 96, 51, 101, 4, 4, 76, 28 },
			{ 0, 103, 46, 99, 4, 4, 84, 20 },
			{ 0, 110, 42, 98, 20, 4, 92, 12 },
			{ 0, 119, 39, 96, 12, 68, 100, 4 },
			{ 0, 121, 47, 88, 20, 60, 4, 4 },
			{ 0, 125, 55, 80, 28, 52, 4, 4 },
			{ 0, 128, 63, 72, 36, 44, 4, 4 },
			{ 0, 133, 71, 63, 4, 36, 4, 4 },
			{ 0, 137, 79, 55, 52, 28, 4, 4 },
			{ 0, 142, 88, 47, 60, 20, 4, 4 },
			{ 0, 148, 96, 39, 68, 12, 4, 4 },
			{ 0, 153, 104, 31, 76, 4, 20, 4 },
			{ 0, 146, 105, 35, 4, 4, 12, 12 },
			{ 0, 141, 107, 40, 4, 4, 4, 20 }
	};



	static int a;
	static int b;
	static int c;
	static int s0;
	static int s1;
	static int s2;
	static int s3;

	int i = 0;
	int position = 1;
	for (int i = 0; i < 48; i ++)
	{
		a = data[i][position];
		position++;
		b = data[i][position];
		position++;
		c = data[i][position];
		position++;
		s0 = data[i][position];
		position++;
		s1 = data[i][position];
		position++;
		s2 = data[i][position];
		position++;
		s3 = data[i][position];
		position = 1;
		
	int x_speed = get_speed(a, b, c, s0, s1, s2, s3, WANT_X);
	int y_speed = get_speed(a, b, c, s0, s1, s2, s3, WANT_Y);

	if (x_speed == GOAL_REACHED && y_speed == GOAL_REACHED)
	{
	printf("GOAL_REACHED \n");
	return 0;
	}
	printf("X_speed=%d Y_speed=%d\n", x_speed, y_speed);
	
	}
	*/
	return 0;


}

// Set goal coordinates from Verification Enviroment.
void set_goal(int goal_x_position, int goal_y_position)
{
#ifdef DEBUG
	printf("SET GOALS: goal_x=%d, goal_y=%d\n",goal_x_position,goal_y_position);
#endif
	X_goal = goal_x_position;
	Y_goal = goal_y_position;
}

int get_speed(int a, int b, int c, int sensor0, int sensor1, int sensor2, int sensor3, int get_value_type)
{

	// static variables for keeping the context of robot position
	static int move_count = 0;
	static int last_direction_X;
	static int last_direction_Y;

	// return speed of robot
	int result = 0;

	// current location of robot
	int spot;



	// X and Y position of the robot
	int *my_position_x, *my_position_y;

	my_position_x = (int *)malloc(sizeof(int));
	my_position_y = (int *)malloc(sizeof(int));

	if ((my_position_x == NULL) || (my_position_y == NULL))
	{
		fprintf(stderr, "Error on memory allocation, exitting...");
		exit(1);
	}

	*my_position_x = x_position(b, c);	
	*my_position_y = y_position(a, b);




	//goal coordinates
	int finish_position_x = X_goal;
	int finish_position_y = Y_goal;

#ifdef DEBUG
	printf("x=%d, y=%d, goal_x=%d, goal_y=%d\n", *my_position_x, *my_position_y, finish_position_x, finish_position_y);
#endif

	// check if robot made it to finish coordinates
	if (*my_position_x == finish_position_x)
		if (*my_position_y == finish_position_y)
		{
		free(my_position_x);
		free(my_position_y);
		return GOAL_REACHED;
		}



	// senzors allocation
	int *s0, *s1, *s2, *s3;

	s0 = (int *)malloc(sizeof(int));
	s1 = (int *)malloc(sizeof(int));
	s2 = (int *)malloc(sizeof(int));
	s3 = (int *)malloc(sizeof(int));

	if ((s0 == NULL) || (s1 == NULL) || (s2 == NULL) || (s3 == NULL))
	{
		fprintf(stderr, "Error on memory allocation, exitting...");
		exit(1);
	}

	// update values according the position of robot
	*s0 = sensor0;
	*s1 = sensor1;
	*s2 = sensor2;
	*s3 = sensor3;


	// specifies the direction which will robot take
	int *direction = (int *)malloc(sizeof(int));
	if (direction == NULL)
	{
		fprintf(stderr, "Error on memory allocation, exitting...");
		exit(1);
	}





	// intial direction which will robot always check
	// if the robot has already taken some moves, use last known direction
	if (move_count == 0 || move_count == 1)
		*direction = EAST;
	else if (get_value_type == WANT_X)
		*direction = last_direction_X;
	else if (get_value_type == WANT_Y)
		*direction = last_direction_Y;


	/* Actual algorithm for computing robot movement in maze*/

	// turn left
	*direction = normalize_direction(*direction - 1);

	// while there is a WALL in my direction...
	while ((spot = check_direction(my_position_x, my_position_y, s0, s1, s2, s3, direction)) == WALL)
	{
		// turn right
		*direction = normalize_direction(*direction + 1);
	}

	// move the robot and get the result speed of movement
	result = move(my_position_x, my_position_y, direction, get_value_type);


	// keeping the context and separating calls for X coordinate and Y coordinate
	if (get_value_type == WANT_X)
		last_direction_X = *direction;
	else
		last_direction_Y = *direction;






	//free the allocated memory



	free(my_position_x);
	free(my_position_y);

	free(s0);
	free(s1);
	free(s2);
	free(s3);

	free(direction);

	// count the moves taken
	move_count++;


	return result;

}




// count the Y coordinate according to the value of position senzors using the Pythagorean theorem
int x_position(int b, int c)
{
	int x_max = x_dimension * box_size;
	int app_x;
	int x;

	app_x = (x_max*x_max - (c*c) + (b*b)) / (2 * x_max);

	x = app_x / box_size;

	return x;
}

// count the X coordinate according to the value of position senzors using the Pythagorean theorem
int y_position(int a, int b)
{
	int y_max = y_dimension * box_size;
	int app_y;
	int y;


	app_y = (y_max*y_max - (a*a) + (b*b)) / (2 * y_max);

	y = app_y / box_size;

	return y;
}

// reversing the axis value in order to match VHDL representation coordinates system
int reverse_x_axis(int x)
{
	int reverse_const = x_dimension;

	return(reverse_const - x);
}



// check if the robot can move to the direction and sets the value of direction variable
int check_direction(int *position_x, int *position_y, int *s0, int *s1, int *s2, int *s3, int *direction)
{

	switch (*direction)
	{
	case WEST:
		if (*s0 < dist_max)
			return WALL;
		else
			return FREE;
		*direction = WEST;
		break;

	case SOUTH:
		if (*s3 < dist_max)
			return WALL;
		else
			return FREE;
		*direction = SOUTH;
		break;

	case EAST:
		if (*s1 < dist_max)
			return WALL;
		else
			return FREE;
		*direction = EAST;
		break;

	case NORTH:
		if (*s2 < dist_max)
			return WALL;
		else
			return FREE;
		*direction = NORTH;
		break;

	default:
		return WALL;
		break;
	}

}

// make direction modular: WEST, NORTH, EAST, SOUTH, WEST, ...
int normalize_direction(int direction)
{
	direction = direction % direction_mod;

	if (direction == -1)
		direction = 3;

	return direction;
}

// move the robot to the specific position according to direction and return speed value
int move(int *position_x, int *position_y, int *direction, int direction_axis)
{
	int X_value = 0;
	int Y_value = 0;

	switch (*direction)
	{
	case WEST:
		*position_y = *position_y - 1;
		X_value = -1;
		Y_value = 0;
		//printf("X_SPEED=-1, Y_SPEED=0\n");
		break;

	case SOUTH:
		*position_x = *position_x + 1;
		X_value = 0;
		Y_value = -1;
		//printf("X_SPEED=0, Y_SPEED=-1\n");
		break;

	case EAST:
		*position_y = *position_y + 1;
		X_value = 1;
		Y_value = 0;
		//printf("X_SPEED=+1, Y_SPEED=0\n");
		break;

	case NORTH:
		*position_x = *position_x - 1;
		X_value = 0;
		Y_value = 1;
		//printf("X_SPEED=0, Y_SPEED=1\n");
		break;
	}

	if (direction_axis == WANT_X)
		return X_value * speed_multiplier;
	else
		return Y_value * speed_multiplier;

}
