/* **********************************************************************************
 * Project Name: Verification Environment for Robot Controller.
 * File Name:    network_comm.c
 * Description:  Set of functions used for network communication with verification enviroment and Player/Stage virtual enviroment.
 * Author:       Stanislav Krajcir <xkrajc06@stud.fit.vutbr.cz>
 * Supervisor:	 Marcela Simkova <isimkova@fit.vutbr.cz>   
 * Date:         25.5.2015
 * **********************************************************************************/


#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <netdb.h>
#include <errno.h>
#include "network_comm.h"


//#define DEBUG

// For maze with dimension 16x16
// static int received_data[8];
// static int data_to_send[3];

// For maze with dimension 8x8
static char received_data[8];
static char data_to_send[3];

int main()

{
	// just for testing purpose
	send_data (123,456);
	receive_data ();
	return 0;
	
}

// Function for sending data data from Verification Enviroment to Player/Stage.
int send_data(int x_speed, int y_speed)
{
  int send_socket, si_other_len;
  struct sockaddr_in si_other;
  struct hostent *hptr;
  int i;
  
  
  if ( (send_socket = socket(PF_INET, SOCK_DGRAM, 0 ) ) < 0) { /* create socket*/
    perror("error on socket");  /* socket error */
    return -1;
  }
  si_other.sin_family = PF_INET;              /*set protocol family to Internet */
  si_other.sin_port = htons(P_S_receive_port);  /* set port no. */
  
  if ( (hptr =  gethostbyname(P_S_host)) == NULL){
    fprintf(stderr, "gethostname error: %s", P_S_host);
    return -1;
   }
  memcpy( &si_other.sin_addr, hptr->h_addr, hptr->h_length);
  
  
 
  si_other_len=sizeof(si_other);
  
  data_to_send[0]=0;
  data_to_send[1]=x_speed;
  data_to_send[2]=y_speed;
	
	if (sendto(send_socket, data_to_send, sizeof(data_to_send), 0, (struct sockaddr*) &si_other, si_other_len) == -1)
		{
			perror("error on sendto");
		}
	
 
    /* close connection, clean up sockets */
   if (close(send_socket) < 0)
   { 
	perror("error on close"); 
	return -1;
   } 
  return 0;
  }

 // Function for receiving data data from Player/Stage.
int receive_data ()
{
  int receive_socket, t, sin_len;
  struct sockaddr_in sin, si_other;
  int i, recv_len;
  
  struct hostent *hptr;

  

 
  if ( (receive_socket = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP ) ) < 0) 
  { /* create socket*/
    perror("error on socket\n");  /* socket error */
    return -1;
  }
  sin.sin_family = PF_INET;              /*set protocol family to Internet */
  sin.sin_port = htons(server_A_port);  /* set port no. */  
  inet_aton(server_A_host, &sin.sin_addr); /* set IP addr to interface */
  
  if (bind(receive_socket, (struct sockaddr *)&sin, sizeof(sin) ) < 0 ) 
  {
    perror("error on bind\n"); return -1;  /* bind error */
  }
 
  sin_len = sizeof(sin);
 
  
	if (recv_len=(recvfrom(receive_socket, received_data, sizeof(received_data), 0, (struct sockaddr *) &sin, &sin_len)) == -1)
		{
			perror("error on receive:");         /*  read error */
			return -1;
		}
#ifdef DEBUG		
	//printf("Received packet from %s:%d\n", inet_ntoa(si_other.sin_addr), ntohs(si_other.sin_port));
	printf("Data:");
	for (i = 0; i < 8; i++)
	{
		printf("%d,",received_data[i]);
	}
	printf("\n");
	
	if (received_data[0] == DATA)
		printf("DATA packet received\n");
	else if (received_data[0] == START)
		printf("START packet received\n");
	else if (received_data[0] == STOP)
		printf("STOP packet received\n");
	else if (received_data[0] == GOAL)
		printf("GOAL packet received\n");
	else
		printf("UNKNOWN packet received\n");
	
#endif	

		

    /* close connection, clean up sockets */
  
   if (close(receive_socket) < 0)
   { 
	perror("error on close"); 
	return -1;
   } 
  return 0;
  }
 
// Return exact value from received data. Context is maintained by static variable.
int get_data()
{
	static int i = 0;
	
	if (i == 7)
		i=0;
	
	i++;
	return received_data[i];
}

