 /* **********************************************************************************
 * Project Name: Verification Environment for Robot Controller.
 * File Name:    rc_env.svh
 * Description:  Class represents Environment for Robot Controller.
 * Author:       Stanislav Krajcir <xkrajc06@stud.fit.vutbr.cz>
 * Supervisor:	 Marcela Simkova <isimkova@fit.vutbr.cz>   
 * Date:         25.5.2015
 * **********************************************************************************/

 class rcEnv extends uvm_env;
    
   //! UVM Factory Registration Macro
   `uvm_component_utils(rcEnv)
   
  /*! 
   * Data Members
   */ 
   
   rcEnvConfig   rc_env_cfg;
   
   
  /*! 
   * Component Members
   */  
   
   rcAgent               rc_agent;
   rcScoreboard          rc_scoreboard;
   
   
  /*!
   * Methods
   */
   
   // Standard UVM methods
   extern function new(string name = "rcEnv", uvm_component parent = null);
   extern function void build_phase(uvm_phase phase);
   extern function void connect_phase(uvm_phase phase);
   
 endclass: rcEnv
 
 

/*! 
 * Constructor - creates the rcEnv object  
 */
 function rcEnv::new(string name = "rcEnv", uvm_component parent = null);
   super.new(name, parent);
 endfunction: new
 

 
 /*! 
 * Build - environment configuration
 */ 
 function void rcEnv::build_phase(uvm_phase phase);
   // check configuration for the Env
   if (!uvm_config_db #(rcEnvConfig)::get(this, "", "rcEnvConfig", rc_env_cfg)) 
     `uvm_error("MYERR", "rcEnvConfig doesn't exist!"); 
   
   rc_agent = rcAgent::type_id::create("rc_agent", this);
   rc_scoreboard = rcScoreboard::type_id::create("rc_scoreboard", this);
   
 endfunction: build_phase
 
 
 
 /*! 
 * Connect - interconnection of environment components
 */ 
 function void rcEnv::connect_phase(uvm_phase phase);	
      // transactions broadcasts to scoreboard
      rc_agent.ap_in.connect(rc_scoreboard.export_rc_in_if);
      rc_agent.ap_out.connect(rc_scoreboard.export_rc_out_if);
	  
 endfunction: connect_phase 