/* **********************************************************************************
 * Project Name: Verification Environment for Robot Controller.
 * File Name:    interface.sv
 * Description:  Interfaces of Robot Controller.
 * Author:       Stanislav Krajcir <xkrajc06@stud.fit.vutbr.cz>
 * Supervisor:	 Marcela Simkova <isimkova@fit.vutbr.cz>   
 * Date:         25.5.2015
 * **********************************************************************************/

import rc_param_pkg::*;

 //Robot Controller Input Interface Declaration

 interface interfaceRCin (input logic CLK, input logic RST);
	logic [DATA_WIDTH-1:0]	DIST_A	;	// distance from point A
	logic [DATA_WIDTH-1:0]	DIST_B	;	// distance from point B	
	logic [DATA_WIDTH-1:0]	DIST_C	;	// distance from point C
	logic [DATA_WIDTH-1:0]	S_0		;	// sensor 0
	logic [DATA_WIDTH-1:0]	S_1		;	// sensor 1
	logic [DATA_WIDTH-1:0]	S_2		;	// sensor 2
	logic [DATA_WIDTH-1:0]	S_3		;	// sensor 3
	logic DATA_VLD				    ;	// enable signal for data valid validity
	
   
   // Clocking block -- synchronizacia na nastupnu hranu hodin
   clocking cb @(posedge CLK);
     //input DATA_VLD;
     output  DIST_A, DIST_B, DIST_C, S_0, S_1, S_2, S_3, DATA_VLD;  
   endclocking: cb;
   


   // DUT Modport
   modport RCin ( input  DIST_A,
				  input  DIST_B,
				  input  DIST_C,
				  input  S_0,
				  input  S_1,
				  input  S_2,
				  input  S_3,
				  input  DATA_VLD 
                 );
  
 endinterface : interfaceRCin
 
 //Robot Controller OUTPUT Interface Declaration
 
 interface interfaceRCout (input logic CLK, input logic RST);
	logic [DATA_WIDTH-1:0]	X_SPEED	;	// speed (value) of X coordinate
	logic [DATA_WIDTH-1:0]	Y_SPEED	;	// speed (value) of Y coordinate
	logic MOVEMENT;						// active while RC moving
	
	
	   // Clocking blocks  
   clocking cb @(posedge CLK);
     input X_SPEED, Y_SPEED, MOVEMENT;   
   endclocking: cb;
   
	// DUT Modport
   modport RCout ( output  X_SPEED,
				   output  Y_SPEED,
				   output  MOVEMENT
				 );
  
 endinterface : interfaceRCout
 
 