 /* **********************************************************************************
 * Project Name: Verification Environment for Robot Controller.
 * File Name:    rc_output_transaction.svh
 * Description:  Class represents transaction which contains values of input signals for Robot Controller.
 * Author:       Stanislav Krajcir <xkrajc06@stud.fit.vutbr.cz>
 * Supervisor:	 Marcela Simkova <isimkova@fit.vutbr.cz>   
 * Date:         25.5.2015
 * **********************************************************************************/
 
 class rcInputTransaction extends uvm_sequence_item;

   //! UVM Factory Registration Macro
   `uvm_object_utils(rcInputTransaction)
   
  /*! 
   * Data Members
   */
   
   logic rst;  // control signals
   
   // values of signals
    logic [DATA_WIDTH-1:0]	dist_a	;	// distance from point A
	logic [DATA_WIDTH-1:0]	dist_b	;	// distance from point B	
	logic [DATA_WIDTH-1:0]	dist_c	;	// distance from point C
	logic [DATA_WIDTH-1:0]	s_0		;	// sensor 0
	logic [DATA_WIDTH-1:0]	s_1		;	// sensor 1
	logic [DATA_WIDTH-1:0]	s_2		;	// sensor 2
	logic [DATA_WIDTH-1:0]	s_3		;	// sensor 3
	logic data_vld					;	// enable signal for data validity

  
  
  /*!
   * Methods
   */ 
  
   // Standard UVM methods
   extern function new(string name = "rcInputTransaction");
   extern function void print(string name);
   extern function void do_copy(uvm_object rhs);
  
 endclass: rcInputTransaction
 
 
 
/*! 
 * Constructor - creates rcInputTransaction object  
 */
 function rcInputTransaction::new(string name = "rcInputTransaction");
   super.new(name);
 endfunction: new



/*! 
 * Implementation of the do_copy() virtual function.
 */
 function void rcInputTransaction::do_copy(uvm_object rhs);
   rcInputTransaction rc_trans;
   
   if(!$cast(rc_trans, rhs)) begin
     uvm_report_error("do_copy:", "$cast failed!");
     return;
   end
   
   super.do_copy(rhs);

   rst     = rc_trans.rst;
   dist_a  = rc_trans.dist_a;
   dist_b  = rc_trans.dist_b; 
   dist_c  = rc_trans.dist_c;
   s_0     = rc_trans.s_0;                
   s_1     = rc_trans.s_1;    
   s_2     = rc_trans.s_2;    
   s_3     = rc_trans.s_3;
   data_vld= rc_trans.data_vld;
   
 endfunction: do_copy  
 


/*! 
 * Print - displays rc Input Transaction content  
 */    
 function void rcInputTransaction::print(string name);
 
 if (name != "") begin
     $write("---------------------------------------------------------\n");
     $write("%s: \n",name);
	 $write("Time = %t ns\n",$time);
     
   end
   
   $write("dist_a: %2d,", dist_a);
   $write("dist_b: %2d,", dist_b);
   $write("dist_c: %2d,", dist_c);
   $write("   s_0: %2d", s_0);
   $write("   s_1: %2d", s_1);
   $write("   s_2: %2d", s_2);
   $write("   s_3: %2d\n", s_3);
   $write("---------------------------------------------------------\n");
   
 
  
 endfunction: print
 
