 /* **********************************************************************************
 * Project Name: Verification Environment for Robot Controller.
 * File Name:    rc_output_transaction.svh
 * Description:  Class represents transaction which contains values of output signals for Robot Controller.
 * Author:       Stanislav Krajcir <xkrajc06@stud.fit.vutbr.cz>
 * Supervisor:	 Marcela Simkova <isimkova@fit.vutbr.cz>   
 * Date:         25.5.2015
 * **********************************************************************************/


 
 class rcOutputTransaction extends uvm_sequence_item;

   //! UVM Factory Registration Macro
   `uvm_object_utils(rcOutputTransaction)
   
  /*! 
   * Data Members
   */
   logic [DATA_WIDTH-1:0]	x_speed	;	// speed (value) of X coordinate
   logic [DATA_WIDTH-1:0]	y_speed	;	// speed (value) of Y coordinate
   logic					movement;	// active while RC is moving

   
  /*!
   * Methods
   */ 
  
   // Standard UVM methods
   extern function new(string name = "rcOutputTransaction");
   extern function void print(string name);
   extern function void do_copy(uvm_object rhs);
   extern function bit do_compare(uvm_object rhs, uvm_comparer comparer);   

   
 endclass: rcOutputTransaction
 
 
 
/*! 
 * Constructor - creates rcOutputTransaction object  
 */
 function rcOutputTransaction::new(string name = "rcOutputTransaction");
   super.new(name);
 endfunction: new



/*! 
 * Implementation of the do_copy() virtual function.
 */
 function void rcOutputTransaction::do_copy(uvm_object rhs);
   rcOutputTransaction rc_trans;
   super.do_copy(rhs);
   if(!$cast(rc_trans, rhs)) begin
     uvm_report_error("do_copy:", "$cast failed!");
     return;
   end
   
   super.do_copy(rhs);
   
   
   x_speed = rc_trans.x_speed;
   y_speed = rc_trans.y_speed;
   movement = rc_trans.movement;
 endfunction: do_copy  
 



/*! 
 * Print - displays RC Input Transaction content  
 */    
 function void rcOutputTransaction::print(string name);
 
   if (name != "") begin
	 $write("---------------------------------------------------------\n");
     $write("%s: \n",name);
	 $write("Time = %t ns\n",$time);
   end
      
   $write("X_SPEED: %2d, Y_SPEED: %2d\n", x_speed,y_speed);
   $write("---------------------------------------------------------\n");
 endfunction: print
 
 /*! 
 * Implementation of the do_compare() virtual function.
 */
 function bit rcOutputTransaction::do_compare(uvm_object rhs, uvm_comparer comparer);
   rcOutputTransaction rc_trans;
   
   if(!$cast(rc_trans, rhs)) begin
     uvm_report_error("do_compare:", "$cast failed!");
     return 0;
   end
   
   return(super.do_compare(rhs, comparer) && (x_speed == rc_trans.x_speed) &&(y_speed == rc_trans.y_speed));
 endfunction: do_compare


