 /* **********************************************************************************
 * Project Name: Verification Environment for Robot Controller.
 * File Name:    transaction_sequence.svh
 * Description:  Class represents UVM sequence of input transactions for Robot Controller.
 * Author:       Stanislav Krajcir <xkrajc06@stud.fit.vutbr.cz>
 * Supervisor:	 Marcela Simkova <isimkova@fit.vutbr.cz>   
 * Date:         25.5.2015
 * **********************************************************************************/
 
 class TransactionSequence extends uvm_sequence #(rcInputTransaction);

   //! UVM Factory Registration Macro
   `uvm_object_utils(TransactionSequence)
  
  /*! 
   * Data Members
   */ 
   
   int trans_count;
   TransactionSequenceConfig transaction_sequence_cfg;
   int output_file;  // output file where transactions are stored
     
  /*!
   * Methods
   */
  
   // Standard UVM methods
   extern function new(string name = "TransactionSequence");
   extern task pre_body();
   extern task body();
   extern task post_body();  
   
   
   // Own UVM methods
   extern task configureSequence(TransactionSequenceConfig transaction_sequence_cfg);
         
 endclass: TransactionSequence
 
 
 
/*! 
 * Constructor - creates TransactionSequence object  
 */
 function TransactionSequence::new(string name = "TransactionSequence");
   super.new(name);
 endfunction: new   



/*! 
 * Pre-body - implements opening of output file where transactions are stored
 */ 
 
 
 task TransactionSequence::pre_body;
 endtask : pre_body



/*! 
 * Body - implements behavior of the transaction
 */ 
 task TransactionSequence::body;
   rcInputTransaction   rc_in_trans, rc_in_trans_c;
   int cnt = 0;
   int i = 0;
   // check configuration for Transaction Sequence
   if (!uvm_config_db #(TransactionSequenceConfig)::get(null, get_full_name(), "TransactionSequenceConfig", transaction_sequence_cfg)) 
     `uvm_error("BODY", "TransactionSequenceConfig doesn't exist!"); 
   
   // configure Sequence of Transactions 
   configureSequence(transaction_sequence_cfg); 
     
   rc_in_trans = rcInputTransaction::type_id::create();

   
  #(RESET_TIME);

	// just for debugging purpose - pooling from Player/Stage
   send_data(123,456);
  

  while (cnt < TRANS_COUNT) begin
     assert($cast(rc_in_trans_c, rc_in_trans.clone));   
     
	 //$write("waiting for data\n");
	/*
	 if (0 != receive_data())
		begin
				// FATAL ERROR
				uvm_report_fatal("RC TRANS_SEQUENCE","Failed on receiving DPI from Player/Stage \n");
		end
	*/
	 receive_data();
	 start_item(rc_in_trans_c);
          
     rc_in_trans_c.dist_a = get_data();
	 rc_in_trans_c.dist_b = get_data();
	 rc_in_trans_c.dist_c = get_data();
	 rc_in_trans_c.s_0 = get_data();
	 rc_in_trans_c.s_1 = get_data();
	 rc_in_trans_c.s_2 = get_data();
	 rc_in_trans_c.s_3 = get_data();
	 rc_in_trans_c.data_vld = 1;
     
     //rc_in_trans_c.print("INPUT TRANSACTION:");
     
     
     finish_item(rc_in_trans_c);
	 if (cnt == 0)
	 begin
		#(INITIAL_WAIT_TIME);
	 end
	 else
	 begin
		#(WAIT_TIME);
	 end
	 
	 cnt++; 
   end 
 endtask: body



/*! 
 * Post-body - implements closing of output file with transactions
 */ 
 task TransactionSequence::post_body;
 endtask : post_body



/*! 
 * configureSequence - configure Sequence with data from the configuration object
 */ 
 task TransactionSequence::configureSequence(TransactionSequenceConfig transaction_sequence_cfg);
   trans_count = transaction_sequence_cfg.trans_count;
 endtask: configureSequence