 /* **********************************************************************************
 * Project Name: Verification Environment for Robot Controller.
 * File Name:    rc_test.svh
 * Description:  Class represents test for Robot Controller.
 * Author:       Stanislav Krajcir <xkrajc06@stud.fit.vutbr.cz>
 * Supervisor:	 Marcela Simkova <isimkova@fit.vutbr.cz>   
 * Date:         25.5.2015
 * **********************************************************************************/


// This class represents UVM test for RC.

 
 class rcTest extends rcTestBase;
    
   //! UVM Factory Registration Macro
   `uvm_component_utils(rcTest)

   /*! 
   * Component Members
   */ 
   TransactionSequence trans_seq;  // sequence of transactions

   /*!
   * Methods
   */
   
   // Standard UVM methods
   extern function new(string name = "rcTest", uvm_component parent = null);
   extern function void build_phase(uvm_phase phase);
   // Other methods
   extern task main_phase(uvm_phase phase);

 endclass: rcTest
 
 
 
/*! 
 * Constructor - creates the RCTest object  
 */
 function rcTest::new(string name = "rcTest", uvm_component parent = null);
   super.new(name, parent);
 endfunction: new



/*! 
 * Build - test configuration
 */ 
 function void rcTest::build_phase(uvm_phase phase);
   super.build_phase(phase);
   
   trans_seq = TransactionSequence::type_id::create("trans_seq");
 endfunction: build_phase



/*! 
 * Main - Stimuli are generated and applied to the DUT
 */     
 task rcTest::main_phase(uvm_phase phase);
    phase.raise_objection(this); 
	
	//print of testing parameters:
   $write("-----------------------------------------------------------\n");
   $write("		TESTING		PARAMETERS 	\n");
   $write("\n");
   $write("CLOCK PERIOD: \t\t%d ns\n", CLK_PERIOD);
   $write("RESET TIME: \t\t\t%d ns\n", RESET_TIME);
   $write("INITIAL WAIT TIME:\t\t%d ns\n", INITIAL_WAIT_TIME);
   $write("WAIT TIME: \t\t\t%d ns\n", WAIT_TIME);
   $write("\n");
   $write("GOAL COORDINATES:\t\t   (%2d,%2d )\n", GOAL_X, GOAL_Y);
   $write("\n");
   $write("TRANSACTION COUNT:\t\t%d\n",TRANS_COUNT);
   $write("TRANSACTION COMPARE COUNT:\t%d\n",TRANSACTION_COMPARE_COUT);
   $write("-----------------------------------------------------------\n");
   $write("VERIFICATION ENVIROMENT WAITING FOR INPUT FROM PLAYER/STAGE\n");
   $write("-----------------------------------------------------------\n");
   
    trans_seq.start(rc_env.rc_agent.trans_sequencer); 
    
    phase.drop_objection(this); 
 endtask: main_phase 