 /* **********************************************************************************
 * Project Name: Verification Environment for Robot Controller.
 * File Name:    rc_test_base.svh
 * Description:  Class represents gemeral test base specification for Robot Controller - General Test Specification.
 * Author:       Stanislav Krajcir <xkrajc06@stud.fit.vutbr.cz>
 * Supervisor:	 Marcela Simkova <isimkova@fit.vutbr.cz>   
 * Date:         25.5.2015
 * **********************************************************************************/


 class rcTestBase extends uvm_test;
    
   //! UVM Factory Registration Macro
   `uvm_component_utils(rcTestBase)
   
  /*!
   * Component Members
   */
   
   rcEnv          rc_env;        // The environment class
  
  /*!
   * Data Members
   */
   
   rcEnvConfig    rc_env_cfg;    // Configuration objects
   rcAgentConfig  rc_agent_cfg;   

  /*!
   * Methods
   */
   
   // Standard UVM methods
   extern function new(string name = "rcTestBase", uvm_component parent = null);
   extern function void build_phase(uvm_phase phase);
   // Other methods
   extern virtual function void configure_env(rcEnvConfig cfg);
   extern virtual function void configure_rc_agent(rcAgentConfig cfg);

 endclass: rcTestBase
 
 
 
/*! 
 * Constructor - creates the RCTestBase object  
 */
 function rcTestBase::new(string name = "rcTestBase", uvm_component parent = null);
   super.new(name, parent);
 endfunction: new



/*! 
 * Build - environment configuration
 */ 
 function void rcTestBase::build_phase(uvm_phase phase);
    
   // CONFIGURATION 
    
   // create configuration object for the RC environment 
   rc_env_cfg = rcEnvConfig::type_id::create("rc_env_cfg");
   
   // configure the RC environment
   configure_env(rc_env_cfg);
   
   // create configuration object for the RC agent
   rc_agent_cfg = rcAgentConfig::type_id::create("rc_agent_cfg");
   
   // configure the RC agent using the agent configuration object
   configure_rc_agent(rc_agent_cfg);
   
   // check virtual interfaces of the RC agent 
   if (!uvm_config_db #(virtual interfaceRCin)::get(this, "", "rcIn_vif", rc_agent_cfg.dut_rc_in_if)) 
     `uvm_error("MYERR", "interfaceRCin:dut_rc_in_if interface doesn't exist!");
   if (!uvm_config_db #(virtual interfaceRCout)::get(this, "", "rcOut_vif", rc_agent_cfg.dut_rc_out_if)) 
     `uvm_error("MYERR", "interfaceRCout:dut_rc_out_if interface doesn't exist!"); 
   
   // the RC Env configuration is saved in the configuration database
   uvm_config_db #(rcEnvConfig)::set(this, "*", "rcEnvConfig", rc_env_cfg);  
     
   // the RC agent configuration is saved in the configuration database
   uvm_config_db #(rcAgentConfig)::set(this, "*", "rcAgentConfig", rc_agent_cfg);  
   
   // CREATE THE RC VERIFICATION ENVIRONMENT
   rc_env = rcEnv::type_id::create("rc_env", this);
   
 endfunction: build_phase



/*! 
 * Configure the RC verification environment.
 */  
 function void rcTestBase::configure_env(rcEnvConfig cfg);
   /*
   ** maybe add something later
   */
 endfunction: configure_env


 
/*! 
 * Configure the RC agent.
 */  
 function void rcTestBase::configure_rc_agent(rcAgentConfig cfg);
   cfg.active                  = UVM_ACTIVE;
   cfg.trans_count             = TRANS_COUNT;

 endfunction: configure_rc_agent 