/* **********************************************************************************
 * Project Name: Verification Environment for Robot Controller.
 * File Name:    top_level.sv
 * Description:  Top Module of Verification Environment for Robot Controller.
 * Author:       Stanislav Krajcir <xkrajc06@stud.fit.vutbr.cz>
 * Supervisor:	 Marcela Simkova <isimkova@fit.vutbr.cz>   
 * Date:         25.5.2015
 * **********************************************************************************/

	module Top_TB;

	import uvm_pkg::*;
	
	import rc_env_pkg::*;
	import rc_test_pkg::*;
	import rc_param_pkg::*;

   //clock and reset signals
   logic CLK;                           
   logic RST;   
  
  
   // virtual interfaces for DUT                      
   interfaceRCin  dut_rc_in_if   (CLK, RST);  // RC input interface
   interfaceRCout dut_rc_out_if  (CLK, RST);  // RC output interface
  
   // DUT
	
   ROBOT_CONTROLLER_ENT #(
     .DATA_WIDTH    (DATA_WIDTH)       
   )
   
   VHDL_DUT_U (
     .CLK            (CLK),
     .RST            (RST),
     
     // Input interface
     .DIST_A            (dut_rc_in_if.DIST_A),
	 .DIST_B            (dut_rc_in_if.DIST_B),
	 .DIST_C            (dut_rc_in_if.DIST_C),
	 .S_0				(dut_rc_in_if.S_0),
	 .S_1				(dut_rc_in_if.S_1),
	 .S_2				(dut_rc_in_if.S_2),
	 .S_3				(dut_rc_in_if.S_3),
	 .DATA_VLD			(dut_rc_in_if.DATA_VLD),
     
     // Output interface
     .X_SPEED       	(dut_rc_out_if.X_SPEED),
     .Y_SPEED     		(dut_rc_out_if.Y_SPEED),
	 .MOVEMENT			(dut_rc_out_if.MOVEMENT)
   );
   
   // UVM initial block
   initial begin
     // virtual interface wrapping - interfaces are put into the configuration database
     uvm_config_db #(virtual interfaceRCin)::set(null, "uvm_test_top", "rcIn_vif", dut_rc_in_if);
     uvm_config_db #(virtual interfaceRCout)::set(null, "uvm_test_top", "rcOut_vif", dut_rc_out_if);
     
	 //run test
	 run_test("rcTest");
	 
     $stop;
   end	// UVM initial block
   
   // Clock and Reset initial blocks
   // Clock generation
   initial begin
     CLK = 0;
     forever begin
       #(CLK_PERIOD/2) CLK = ~CLK;	//set clock period in separate package file
     end
   end

   // Reset at the start of the simulation
   initial begin
     RST = 1;
     #(RESET_TIME);
     RST = 0;

   end 
  
 endmodule: Top_TB